package com.infovis.callgraph;

import graph.CompoundGraph;
import graph.HierarchyGraph.HNode;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingConstants;

import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.*;
import org.eclipse.swt.SWT;


import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;

import renderer.RenderThread;

/**
 * A view.
 * 
 * Abstract Syntax Tree
 * 
 * http://www.eclipse.org/articles/article.php?file=Article-JavaCodeManipulation_AST/index.html
 * 
 * http://www.vogella.com/articles/EclipseJDT/article.html
 * 
 * (http://www.eclipse.org/forums/index.php/m/835113/ =>)
 * http://help.eclipse.org/helios/index.jsp?topic=%2Forg.eclipse.jdt.doc.isv%2Fguide%2Fjdt_api_search.htm
 * 
 * 
 * @author DarkMatt
 *
 */
public class CallGraphView extends ViewPart implements ActionListener, ItemListener, CallBackInterface {

	private static final String FILEEXTENSION = "java";
	
	private Composite composite;
	private Canvas canvas;
	private JLabel outputString = new JLabel("0 active edges");
	private JComboBox<String> projectList;
	private JComboBox<String> sizeList;
	private JComboBox<String> splineList;
	private JProgressBar progressBar;
	private JCheckBox contextCheckBox;
	
	private CompoundGraph cGraph;
	
	private RenderThread renderer;
	
	/**
	 * Starts the renderer.
	 */
	private void initGL() {
		
		renderer = new RenderThread(canvas, 500, 250, this);
		renderer.start();
		
	}
	
	/**
	 * On initialize creates the user swing user interface.
	 */
	public void createPartControl(Composite parent) {
		
		// add a canvas
		composite = new Composite(parent, SWT.EMBEDDED);
		Frame frame = SWT_AWT.new_Frame(composite);
		frame.setLayout(new BorderLayout()); 
		canvas = new Canvas();
		canvas.setFocusable(true);
		frame.add(canvas, BorderLayout.CENTER);
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		JPanel leftPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(leftPanel);
		JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panel.add(rightPanel);
		
		initGL();
		
		String[] projects = init(0);
		projectList = new JComboBox<String>(projects);
		projectList.setSelectedIndex(0);
		leftPanel.add(projectList);
		
		JButton refreshButton = new JButton("Refresh");
		refreshButton.addActionListener(this);
		refreshButton.setActionCommand("refresh");
		leftPanel.add(refreshButton);

		leftPanel.add(outputString);
		leftPanel.add(Box.createHorizontalStrut(40));
		
		leftPanel.add(new JLabel("Text size: "));
		sizeList = new JComboBox<String>(new String[]{"small", "medium", "large"});
		sizeList.addActionListener(this);
		sizeList.setActionCommand("setText");
		leftPanel.add(sizeList);
		
		leftPanel.add(new JLabel("Spline thickness: "));
		splineList = new JComboBox<String>(new String[]{"thin", "medium", "thick"});
		splineList.setSelectedIndex(1);
		splineList.addActionListener(this);
		splineList.setActionCommand("setSpline");
		leftPanel.add(splineList);
		
		contextCheckBox = new JCheckBox("Context");
		contextCheckBox.setHorizontalTextPosition(SwingConstants.LEFT);
		contextCheckBox.setSelected(false);
		contextCheckBox.addItemListener(this);
		leftPanel.add(contextCheckBox);
		
		progressBar = new JProgressBar(0, 100);
		progressBar.setValue(100);
		progressBar.setStringPainted(true);
		rightPanel.add(progressBar);
		
		frame.add(panel, BorderLayout.SOUTH);
		
	}
	
	/**
	 * Initializes the data from the eclipse projects.
	 * 
	 * @param project
	 * @return
	 */
	private String[] init(int project) {
		
		//String msg = "";
		
		// workspace
		IWorkspaceRoot myWorkspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		// projects
		IProject projects[] = myWorkspaceRoot.getProjects();
		
		if (projects.length > project) {
			processProject(projects[project]);
		}
		
		//System.out.println(msg);
		
		String[] out = new String[projects.length];
		
		for (int i = 0; i < projects.length; i++) {
			out[i] = projects[i].getName();
		}
		
		return out;
		
	}
	
	/**
	 * Returns the project name.
	 * 
	 * @param project
	 * @return
	 */
	private String processProject(IProject project) {
		
		if (!project.isOpen()) try { project.open(null); } catch (CoreException e) { e.printStackTrace(); } // open project if not opened
		
		cGraph = new CompoundGraph();
		
		// generate hierarchy graph!
		System.out.println("--------------------------");
		System.out.println("Generating Hierarchy Graph");
		System.out.println("--------------------------");
		
		IFolder srcFolder = project.getFolder("src");
		HNode root = cGraph.addHierarchyRoot(srcFolder.getName());
		addNodesToGraph(srcFolder, root);
		
		cGraph.layoutHierarchyGraph();
		
		float[] points = cGraph.getHierarchyPoints();
		String[] labels = cGraph.getHierarchyPointLabels();
		
		System.out.println("");
		
		System.out.println("---------------------------");
		System.out.println("Generating Connection Graph");
		System.out.println("---------------------------");
		
		// create java project using the Abstract Syntax Tree
		IJavaProject jProject = JavaCore.create(project);
		
		// gets all the packages
		IPackageFragment[] packages = null;
		try {
			packages = jProject.getPackageFragments();
		} catch (JavaModelException e) {
			e.printStackTrace();
		}
		
		// count 
		int count = 0;
		for (int i = 0; i < packages.length; i++) {
			try {
				// only process source packages!
				if (packages[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
					count++;
				}
			} catch (JavaModelException e) {
				e.printStackTrace();
			}
		}
		
		if (progressBar != null) progressBar.setMaximum(count); // not on starting up
		
		// outputs all packages
		count = 0;
		for (int i = 0; i < packages.length; i++) {
			try {
				// only process source packages!
				if (packages[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
					getPackages(packages[i]);
				}
				if (progressBar != null) {
					progressBar.setValue(count++); // not on starting up
					progressBar.update(progressBar.getGraphics());
				}
			} catch (JavaModelException e) {
				e.printStackTrace();
			}
		}
		
		//ArrayList<ConnectionEdge> conn = cGraph.getEdges();
		renderer.setGraph(points, labels, cGraph.getEdges(), cGraph.getHierarchyEdges());
		
		return project.getName();
	}
	
	/**
	 * Gets the packages of a projects and processes further.
	 * 
	 * @param p
	 */
	private void getPackages(IPackageFragment p) {
		
		/*System.out.println();
		System.out.println("Package name: " + p.getElementName());
		System.out.println("--------------------------------------");*/
		
		ArrayList<String> path = new ArrayList<String>();
		path.add("src");														// TODO HACK
		String directory = p.getElementName();
		String[] directories = directory.split("[\\.]");
		Collections.addAll(path, directories);
		
		for (int i = 0; i < path.size(); i++) { // remove empty
			if (path.get(i).equals("")) {
				path.remove(i);
				i --;
			}
		}
		
		try {
			ICompilationUnit[] units = p.getCompilationUnits();
			
			
			for (int i = 0; i < units.length; i++) {
				getClasses(units[i], new ArrayList<String>(path));
			}
			
		} catch (JavaModelException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Gets the classes of a package and processes further.
	 * 
	 * @param unit
	 * @param path
	 */
	private void getClasses(ICompilationUnit unit, ArrayList<String> path) {
		
		try {
			
			IType[] types = unit.getAllTypes();
			
			for (IType type : types) {
				
				//Document doc = new Document(unit.getSource());
				
				// file-name
				//System.out.println( "File name: " + type.getElementName() + " with lines of code " + doc.getNumberOfLines());
				
				path.add(type.getElementName());
				
				getMethods(type, new ArrayList<String>(path));
				
			}
			
		} catch (JavaModelException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Required for a search request - executed for every found method.
	 */
	private class SearchImplementation extends SearchRequestor {

		private boolean isFinished;
		private ArrayList<ArrayList<String>> results;
		
		private void addResult(ArrayList<String> nRes) {
			results.add(nRes);
		}
		
		public Integer getResultsSize() {
			if (!isFinished) {
				return null;
			} else {
				return results.size();
			}
		}
		
		public String[] getResult(int index) {
			return results.get(index).toArray(new String[results.get(index).size()]);
		}
		
		@Override
		public void beginReporting() {
			super.beginReporting();
			results = new ArrayList<ArrayList<String>>();
			isFinished = false;
		}
		
		@Override
		public void endReporting() {
			super.endReporting();
			isFinished = true;
		}
		
		@Override
		public void acceptSearchMatch(SearchMatch match) throws CoreException {
			
			if (match.getElement() instanceof IMethod) {
			
				IMethod caller = (IMethod)match.getElement();
				//System.out.println("Call from " + caller.getElementName() + " in " + caller.getParent().getElementName());
				
				//caller.getElementName() // is the method name that calls the other method
				
				ArrayList<String> newResult = new ArrayList<String>();
				
				String parent = "";
				
				IJavaElement ele = caller.getParent().getParent();
				while((ele) != null) {
					if (!ele.getElementName().equals("")) {
						parent = ele.getElementName() + "/" +  parent;
						
						String name = ele.getElementName();
						
						String[] subStrings = name.split("\\.");
						// remove .java	
						if (subStrings[subStrings.length-1].equals("java")) {
							subStrings = Arrays.copyOf(subStrings, subStrings.length-1);
						}
						for (int i = subStrings.length-1; i >= 0; i--) {
							newResult.add(subStrings[i]);
						}
						//newResult.add(name);			// this is inverted! starts at the class level
					}
					ele = ele.getParent();
				}
				parent = parent.substring(0, parent.length()-6);
				
				//System.out.println("Called by " + parent);
				
				this.addResult(newResult);
				
			}
			
		}
		
	}
	
	/**
	 * Iterates over the methods of a class and finds the call hierarchy.
	 * 
	 * @param type
	 */
	private void getMethods(IType type, ArrayList<String> path) {
		
		try {
			IMethod[] methods = type.getMethods();
			
			for (IMethod method : methods) {
				
				//System.out.println("Method name " + method.getElementName());
				// getParameterNames()
				//System.out.println("Return Type " + method.getReturnType());
				// isConstructor() 
				
				/*String o = "";
				for (int i = 0; i < path.size(); i++) {
					o += path.get(i) + "/";
				}
				o += method.getElementName();*/
				//System.out.println(o);
				
				// FINDING CALL HIERARCHY
				
				// search pattern
				SearchPattern pattern = SearchPattern.createPattern(method, IJavaSearchConstants.REFERENCES);

			    // search scope
			    IJavaSearchScope scope = SearchEngine.createWorkspaceScope();
				
			    // search result
			    SearchImplementation requestor = new SearchImplementation(); // TODO INIT?
				
			    // search
			    SearchEngine searchEngine = new SearchEngine();
			    searchEngine.search(pattern, new SearchParticipant[] {SearchEngine.getDefaultSearchParticipant()}, scope, requestor, null);
			    
			    /*while (!requestor.finishedSearch()) {
			    	
			    	String[] result = null;
				    while (result == null) {
				    	System.out.println("in while");
				    	result = requestor.result();
					}
				    
				    cGraph.addConnectionEdge(result, path.toArray(new String[path.size()]));
				    
				    System.out.println("added edge...");
				    
			    }*/
			    Integer size;
			    while ((size = requestor.getResultsSize()) == null) {
					// wait
				}
			    
			    for (int i = 0; i < size; i++) {
			    	cGraph.addConnectionEdge(method.getElementName(), requestor.getResult(i), path.toArray(new String[path.size()]));
				}
			    
			    //System.out.println("Finished.");
			    
			    /*String[] result = null;
			    while (result == null) {
			    	result = requestor.result();
				}
			    
			    cGraph.addConnectionEdge(result, path.toArray(new String[path.size()]));*/
			    
			}
			
		} catch (JavaModelException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Adds nodes to the compound graph.
	 * 
	 * @param rootFolder
	 * @param parent
	 */
	private void addNodesToGraph(IFolder rootFolder, HNode parent) {
		
		/**
		 * Information in use
		 * http://cos-mcs.googlecode.com/svn-history/r216/trunk/generation/src/org/cos/generation/plugin/GenerateModelAction.java
		 */
		
		IResource[] files = null;
		try {
			files = rootFolder.members();
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
		for (int i = 0; i < files.length; i++) {
			if (files[i].getType() == IResource.FILE && files[i].getFileExtension().equals(FILEEXTENSION)) {
				cGraph.addHierarchyNode(parent, files[i].getName());
			}
			if (files[i].getType() == IResource.FOLDER) {
				HNode nParent = cGraph.addHierarchyNode(parent, files[i].getName());
				addNodesToGraph((IFolder)files[i], nParent);
			}
		}
	}
	
	/**
	 * Necessary empty method.
	 */
	public void setFocus() {
		
	}

	/**
	 * User interface callback.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("refresh")) {
			init(projectList.getSelectedIndex());
		} else if (e.getActionCommand().equals("setText")) {
			int index = sizeList.getSelectedIndex();
			switch (index) {
				case 0:
					renderer.setTextSize(0.12f);
					break;
				case 1:
					renderer.setTextSize(0.15f);
					break;
				case 2:
					renderer.setTextSize(0.2f);
					break;
	
				default:
					break;
			}
			
		} else if (e.getActionCommand().equals("setSpline")) {
			int index = splineList.getSelectedIndex();
			switch (index) {
				case 0:
					renderer.setSplineThickness(1.25f);
					break;
				case 1:
					renderer.setSplineThickness(1.75f);
					break;
				case 2:
					renderer.setSplineThickness(2.25f);
					break;
	
				default:
					break;
			}
			
		}
	}

	/**
	 * Callback implementation
	 */
	@Override
	public void setActiveEdgeCount(int count) {
		outputString.setText(count + " active edges");
	}
	
	/**
	 * Callback implementation
	 */
	@Override
	public void setActiveText(String text) {
		outputString.setText(text);
	}

	/**
	 * Callback implementation
	 */
	@Override
	public void setStatus(int current, int maximum) {
		progressBar.setValue(current);
		progressBar.setMaximum(maximum);
	}

	/**
	 * User interface callback implementation
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
		Object source = e.getItemSelectable();
		if (source == contextCheckBox) {
			renderer.setSplineContext(contextCheckBox.isSelected());
		}
	}
	
}