package graph;

import java.util.ArrayList;

import graph.HierarchyGraph.HNode;

/**
 * 
 * Compound graph structure.
 * 
 * @author DarkMatt
 *
 */

public class CompoundGraph {
	
	/**
	 * Hierarchy graph part of the compound graph.
	 */
	private HierarchyGraph hierarchy;
	/**
	 * Connections (edges) in the compound graph.
	 */
	private ConnectionList connections;
	
	/**
	 * Constructor.
	 */
	public CompoundGraph() {
		hierarchy = new HierarchyGraph();
		connections = new ConnectionList();
	}
	
	/**
	 * Adds the root of the hierarchy graph.
	 * 
	 * @param root
	 */
	public HNode addHierarchyRoot(String root) {
		return hierarchy.addRoot(root);
	}
	
	/**
	 * Adds a node to the hierarchy graph.
	 * 
	 * @param root
	 * @param node
	 */
	public HNode addHierarchyNode(HNode root, String node) {
		return hierarchy.addNode(root, node);
	}
	
	/**
	 * Positions the hierarchy graph.
	 */
	public void layoutHierarchyGraph() {
		hierarchy.reArrangeGraph();
		hierarchy.layoutGraph(new float[]{0.0f, 0.5f});
	}
	
	/**
	 * Returns the nodes positions of the hierarchy graph.
	 * 
	 * @return
	 */
	public float[] getHierarchyPoints() {
		return hierarchy.getPoints();
	}
	
	/**
	 * Returns the text description of every hierarchy graph node.
	 * 
	 * @return
	 */
	public String[] getHierarchyPointLabels() {
		return hierarchy.getPointLabels();
	}
	
	/**
	 * Adds a connection edge to the compound graph.
	 * 
	 * @param methodName
	 * @param nodesCaller
	 * @param nodesCallee
	 */
	public void addConnectionEdge(String methodName, String[] nodesCaller, String[] nodesCallee) {
		connections.addConnectionEdge(methodName, nodesCaller, nodesCallee, hierarchy);
	}
	
	/**
	 * Returns all edges of the compound graph.
	 * 
	 * @return
	 */
	public ArrayList<ConnectionEdge> getEdges() {
		return connections.getEdges();
	}
	
	/**
	 * Returns the edges between the hierarchy structure of the compound graph.
	 * 
	 * @return
	 */
	public ArrayList<ConnectionEdge> getHierarchyEdges() {
		return hierarchy.getEdges();
	}
	
}
