package graph;

import graph.HierarchyGraph.HNode;

import java.util.ArrayList;

/**
 * The connection structure of the compound graph. 
 * (List structure for edges in the compound graph.)
 * 
 * @author DarkMatt
 *
 */

public class ConnectionList {
	
	/**
	 * List of edges.
	 */
	private ArrayList<ConnectionEdge> edges;
	
	/**
	 * Constructor.
	 */
	public ConnectionList() {
		edges = new ArrayList<ConnectionEdge>();
	}
	
	/**
	 * Adds an edge. The input are two string arrays with node names.
	 * 
	 * Caller: caller.class/.../src
	 * Callee: src/.../callee.class
	 * 
	 * @param nodesStart
	 * @param nodesEnd
	 */
	public void addConnectionEdge(String methodName, String[] nodesCaller, String[] nodesCallee, HierarchyGraph hierarchy) {
		ConnectionEdge toAdd = new ConnectionEdge();
		toAdd.name = methodName;
		
		// a directed edge from caller to callee
		
		String nearestElement = "";
		// find the shortest path
		for (int i = 0; i < nodesCallee.length; i++) {
			
			String currentCallee = nodesCallee[i];
			
			for (int j = 0; j < nodesCaller.length; j++) {
				if (currentCallee.equals(nodesCaller[j])) {
					nearestElement = currentCallee;
				}
			}
		}
		
		/*if (nodesCallee[nodesCallee.length-1].equals("SceneObject")) {
			System.out.println("bla");
		}*/
		
		//System.out.println("Nearest element is " + nearestElement + " caller " + nodesCaller[0] + " callee " + nodesCallee[nodesCallee.length-1]);
		
		// given: src/.../callee.class
		// given: caller.class/.../src
		// wanted result => caller.class/.../nearestElement/.../callee.class
		
		ArrayList<String> pathNames = new ArrayList<String>();
		ArrayList<Float> path = new ArrayList<Float>();
		
		// fill inverse caller side - why the hell is this inverted?
		for (int i = 0; i < nodesCaller.length; i++) {
			if (!nodesCaller[i].equals(nearestElement)) {
				HNode current = hierarchy.findNode(nodesCaller[i]);
				if (current != null) {
					float[] pos = current.position;
					path.add(pos[0]);
					path.add(pos[1]);
					pathNames.add(nodesCaller[i]);
				}
				
			} else {
				break;
			}
		}
		// nearest element
		float[] pos = hierarchy.findNode(nearestElement).position;
		path.add(pos[0]);
		path.add(pos[1]);
		pathNames.add(hierarchy.findNode(nearestElement).name);
		// fill callee side
		boolean fill = false;
		for (int i = 0; i < nodesCallee.length; i++) {
			if (fill) {
				HNode current = hierarchy.findNode(nodesCallee[i]);
				if (current != null) {
					pos = current.position;
					path.add(pos[0]);
					path.add(pos[1]);
					pathNames.add(nodesCallee[i]);
				}
			}
			if (nodesCallee[i].equals(nearestElement)) {
				fill = true;
			}
		}
		
		float[] pathA = new float[path.size()];
		for (int i = 0; i < pathA.length; i++) {
			pathA[i] = path.get(i);
		}
		
		String[] pathStr = new String[pathNames.size()];
		for (int i = 0; i < pathStr.length; i++) {
			pathStr[i] = pathNames.get(i);
		}
		
		toAdd.path = pathA;
		toAdd.nodes = pathStr;
		
		edges.add(toAdd);
	}
	
	/**
	 * Returns the edges list.
	 * 
	 * @return
	 */
	public ArrayList<ConnectionEdge> getEdges() {
		return edges;
	}
	
}
