package interaction;

import java.util.ArrayList;

import org.lwjgl.opengl.GL11;

/**
 * For clicking nodes.
 * 
 * @author DarkMatt
 *
 */
public class CollisionDetection {

	/**
	 * Bounding box.
	 * 
	 * @author DarkMatt
	 *
	 */
	private class BoundingInformation {
		public float[] bb;
		public BoundingInformation(float[] boundingArea) {
			this.bb = boundingArea;
		}
	}
	/**
	 * List of all bounding bodes, this corresponds to the indices (sorting) of the nodes.
	 */
	private ArrayList<BoundingInformation> boudingAreas;
	
	/**
	 * Constructor.
	 */
	public CollisionDetection() {
		reset();
	}
	
	/**
	 * Reset to be called on loading a new graph or reloading the old one.
	 */
	public void reset() {
		boudingAreas = new ArrayList<BoundingInformation>();
	}
	
	/**
	 * Adds a single bounding element.
	 * 
	 * @param area
	 */
	public void addElement(float[] area) {
		boudingAreas.add(new BoundingInformation(new float[]{area[0], area[1], area[0]+area[2], area[1]+area[3]}));
	}
	
	/**
	 * Calls a collision into the elements and returns the colliding index.
	 * 
	 * @param pos
	 * @param scale
	 * @return
	 */
	public int getCollisionIndex(float[] pos, float[] scale) {
		
		int index = -1;
		
		for (int i = 0; i < boudingAreas.size(); i++) {
			float[] b = boudingAreas.get(i).bb;
			if (b[0] < pos[0] && b[2] > pos[0] && b[1] < pos[1] && b[3] > pos[1]) {
				index = i;
			}
		}
		
		return index;
	}
	
	/**
	 * Debug drawing of the boundingboxes.
	 */
	public void drawBB() {
		
		//GL11.glMatrixMode(GL11.GL_MODELVIEW);
		//GL11.glLoadIdentity();
		
		//GL11.glPushMatrix();
		
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		
		GL11.glColor3f(1.0f, 0.0f, 0.0f);
		for (int i = 0; i < boudingAreas.size(); i++) {
			float[] p = boudingAreas.get(i).bb;
			GL11.glBegin(GL11.GL_LINE_LOOP);
				GL11.glVertex3f(p[0], 	p[1],		0.0f);
				GL11.glVertex3f(p[2],	p[1],		0.0f);
				GL11.glVertex3f(p[2], 	p[3],	0.0f);
				GL11.glVertex3f(p[0], 	p[3],	0.0f);
		    GL11.glEnd();
		}
		
		//GL11.glPopMatrix();
	}
	
}
