package renderer;

import java.util.ArrayList;

import renderer.shader.ShaderCollection;

/**
 * a colelction of splines.
 * 
 * @author DarkMatt
 *
 */

public class SplineCollection {

	private static final float BETA_OFFSET = 0.002f;
	private float beta = 1.0f;
	
	private static final int SPLINE_SIZE = 15;
	protected ArrayList<CubicBSpline> splines;
	
	/**
	 * Constructor
	 */
	public SplineCollection() {
		splines = new ArrayList<CubicBSpline>();
	}
	
	/**
	 * Adds a spline with 2D control points.
	 * 
	 * @param points
	 * @param textures
	 * @param vbo
	 */
	public void add2D(float[] points, TextureCollection textures, VBO vbo) {
		
		CubicBSpline toAdd = new CubicBSpline();
		toAdd.loadControlPoints2D((beta-=BETA_OFFSET), points, SPLINE_SIZE, textures, vbo);
		
		if (beta < 0.2f) {
			beta = 1.0f;
		}
		
		//System.out.println("SplineCollection: " + beta);
		
		splines.add(toAdd);
		
	}
	
	/**
	 * Draws all splines.
	 * 
	 * @param viewMatrix
	 * @param textures
	 * @param shaders
	 * @param vbo
	 * @return
	 */
	public int drawAll(float[] viewMatrix, TextureCollection textures, ShaderCollection shaders, VBO vbo) {
		// draw splines
		for (int j = 0; j < splines.size(); j++) {
			splines.get(j).drawSpline(viewMatrix, textures, shaders, vbo, 1.0f);
		}
		
		return splines.size();
	}
	
	/**
	 * Sets thickness for all splines.
	 * 
	 * @param thickness
	 */
	public void setSplineThickness(float thickness) {
		for (int j = 0; j < splines.size(); j++) {
			splines.get(j).setSplineThickness(thickness);
		}
	}
	
}
