package renderer;

import java.util.ArrayList;

import renderer.shader.ShaderCollection;

/**
 * Spline collection that allows to draw only specific splines.
 * 
 * @author DarkMatt
 *
 */

public class SplineCollectionSelectable extends SplineCollection {
	
	private boolean FC = false;
	
	private ArrayList<String[]> labels;
	
	/**
	 * Constructor
	 */
	public SplineCollectionSelectable() {
		super();
		labels = new ArrayList<String[]>();
	}
	
	/**
	 * Add 2D spline control points with a String array of nodes which the spline goes through.
	 * 
	 * @param labels
	 * @param points
	 * @param textures
	 * @param vbo
	 */
	public void add2D(String[] labels, float[] points, TextureCollection textures, VBO vbo) {
		this.labels.add(labels);
		super.add2D(points, textures, vbo);
	}
	
	/**
	 * Draw a selection of splines going through a certain node specified with a String.
	 * 
	 * @param label
	 * @param viewMatrix
	 * @param textures
	 * @param shaders
	 * @param vbo
	 * @return
	 */
	public int drawAll(String label, float[] viewMatrix, TextureCollection textures, ShaderCollection shaders, VBO vbo) {
		// draw splines
		int count = 0;
		for (int i = 0; i < splines.size(); i++) {
			boolean draw = false;
			for (int j = 0; j < labels.get(i).length; j++) {
				if (labels.get(i)[j].equals(label)) {
					draw = true;
				}
			}
			if (draw) {
				count ++;
				splines.get(i).drawSpline(viewMatrix, textures, shaders, vbo, 1.0f);
			} else if (FC) {
				splines.get(i).drawSpline(viewMatrix, textures, shaders, vbo, 0.0f);
			}
		}
		return count;
	}
	
	/**
	 * Enables the focus and context (grey out) of splines.
	 * 
	 * @param context
	 */
	public void setEnableContext(boolean context) {
		FC = context;
	}
	
}
