package tools;

import org.lwjgl.util.vector.Matrix4f;

public class Matrix extends Matrix4f {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Identity matrix.
	 * 
	 * @return
	 */
	public static float[] identity() {
		return new float[]{
				1.0f, 0.0f, 0.0f, 0.0f,
				0.0f, 1.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 1.0f, 0.0f,
				0.0f, 0.0f, 0.0f, 1.0f};
	}
	
	/**
	 * Orthogonal projection matrix.
	 * 
	 * @param sizeX
	 * @param sizeY
	 * @param zNear
	 * @param zFar
	 * @return
	 */
	public static float[] orthogonal(float sizeX, float sizeY, float zNear, float zFar) {
		
		/*//TEST
		GL11.glOrtho(-1*sizeX, 1*sizeX, -1*sizeY, 1*sizeY, zNear, zFar);
		
		float Minv[] = new float[16];
		ByteBuffer temp = ByteBuffer.allocateDirect(64);
		temp.order(ByteOrder.nativeOrder());
		GL11.glGetFloat(GL11.GL_MODELVIEW_MATRIX, (FloatBuffer)temp.asFloatBuffer());
		temp.asFloatBuffer().get(Minv);*/
		
		float[] out = identity();
			
		out[0] = 1.0f/sizeX;
		out[5] = 1.0f/sizeY;
		out[10] = 2/(zNear-zFar);
		out[11] = (zFar+zNear)/(zNear-zFar);
		
		return out;
		
	}
	
	/**
	 * Perspective projection matrix.
	 * 
	 * @param fovy
	 * @param aspect
	 * @param zNear
	 * @param zFar
	 * @return
	 */
	public static float[] perspective(float fovy, float aspect, float zNear, float zFar) {
		
		float[] out = identity();
		
		float f = 1.0f / (float)Math.tan(Math.toRadians(fovy)/2.0f);
			
		out[0] = f/aspect;
		out[5] = f;
		out[10] = (zFar+zNear)/(zNear-zFar); 	//why is this not negative? - this is the correct implementation!
		out[11] = (2*zFar*zNear)/(zNear-zFar);	//why is this not negative? - this is the correct implementation!
		out[14] = -1.0f;
		out[15] = 0.0f;
		
		return out;
		
	}
	
	/**
	 * Matrix to array.
	 * 
	 * @param m
	 * @return
	 */
	public static float[] matrixToArray(Matrix4f m) {
		return new float[]
		                 {
							m.m00, m.m10, m.m20, m.m30,
							m.m01, m.m11, m.m21, m.m31,
							m.m02, m.m12, m.m22, m.m32,
							m.m03, m.m13, m.m23, m.m33
		                 };
	}
	
	/**
	 * Matrix to array.
	 * 
	 * @return
	 */
	public float[] getMatrix() {
		return new float[]
		                 {
							this.m00, this.m10, this.m20, this.m30,
							this.m01, this.m11, this.m21, this.m31,
							this.m02, this.m12, this.m22, this.m32,
							this.m03, this.m13, this.m23, this.m33,
		                 };
	}
	
	/**
	 * Transpose matrix to array.
	 * 
	 * @return
	 */
	public float[] getTransposeMatrix() {
		return new float[]
		                 {
							this.m00, this.m01, this.m02, this.m03,
							this.m10, this.m11, this.m12, this.m13,
							this.m20, this.m21, this.m22, this.m23,
							this.m30, this.m31, this.m32, this.m33,
		                 };
	}
	
}
